@@ -972,7 +972,6 @@ _ecore_evas_wayland_egl_resize(Ecore_Evas *ee, int location)
                ERR("evas_engine_info_set() for engine '%s' failed.", ee->driver);
           }
 
-        ee->engine.wl.win->resizing = EINA_TRUE;
         ecore_wl_window_resize(ee->engine.wl.win, ee->w, ee->h, location);
      }
 }
@@ -985,7 +984,6 @@ _ecore_evas_wayland_egl_move(Ecore_Evas *ee, int x, int y)
    if (!ee) return;
    if (ee->engine.wl.win) 
      {
-        ee->engine.wl.win->moving = EINA_TRUE;
         ecore_wl_window_move(ee->engine.wl.win, x, y);
      }
 }
